#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define RESETX 0 //-1100  //start here. For testing, change to negative x value and start terrain there
#define RESETY SCREEN_HEIGHT - lander_H - 2 //24 //SCREEN_HEIGHT - lander_H - 2
#define STARTSTATE RESET
bool pupsLoaded = false;


int upKey = 20;

#define RESET 1
#define MOVEUP 2
#define MOVEDOWN 3
#define EXPLODE 4
#define LANDED 5
#define LOADPUPS 6
#define UNLOADPUPS 7

#define COLLNONE 0
#define COLLGROUND 1
#define COLLMETEOR 2
#define COLLFUEL 3
#define COLLLANDED 4




#include "terrainAll.h"
#include "expAll.h"
#include "icons.h"

#define MAXFUEL 400
#define MAXHEAT 30
#define OFFSCREENX 148
#define OVERHEATMAX 5


int gameState = STARTSTATE;

int landedY = 12 + lander_H;
int vLoc = RESETY;
int up = 5;

int spd = 30;
String fuel;
bool moveTerr;
bool resetTerr;
int terrX = RESETX;// set RESETX to 0 for full play, set to another (where the terrain doesn't interfere) value for testing
unsigned long terrX_T = 0;
int terrMvT = 100;


float heatF = 10;
int heat = heatF;
bool overHeat = false;
bool flashHeat = false;
unsigned long flashHeatT;
int overHeatCnt = OVERHEATMAX;

int fuelAmnt = MAXFUEL;
int fuelX = 30;
int fuelH = 8;

int meteorX = OFFSCREENX;
unsigned long meteorT;
int meteorSpd = 10;
bool meteorGo = false;

int refuelX = OFFSCREENX;
unsigned long refuelT;
int refuelSpd = 5;
bool refuelGo = false;


int collision = COLLNONE;
int expLState = 0;
unsigned long expLT, expLTnoPlay;
int expLFrame = 40;

bool landNow = false;
bool landOk = false;


int pupX[8], pupY, dist;

void setup() {
  Serial.begin(115200);
  //while(!Serial);
  pinMode(upKey,INPUT_PULLUP);
  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS);
  // Clear the buffer
  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);

}


void loop() {


  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);
  display.setTextSize(1);            
  display.setCursor(0, 0);

  display.println("FUEL");
  display.fillRect(fuelX,0,fuelAmnt/10,fuelH,SSD1306_WHITE); 
  display.setCursor(80, 0);

  if (flashHeat){
    if (millis() > flashHeatT + 200){
      dispHeat();
      flashHeatT = millis();
    }
  }
  else dispHeat();



  switch(gameState){
    case RESET://game reset
      terrain(moveTerr);

      display.setCursor(20, 16);
      if (!pupsLoaded){
        display.println("To Save Moon");
        display.println("Puppies press button");
      }else{
        vLoc = 63 - landedY;  
        display.println("TAKEOFF!");
      }
      drawLander(); 

      if (!digitalRead(upKey)){//
        gameState = MOVEUP;
        expLTnoPlay = millis();
      }
    break;
    case MOVEDOWN://Lander is descending or sitting at beginning waiting for an up command
      if (!digitalRead(upKey) && fuelAmnt > 0){
        if (overHeatCnt > 0 && heat < 25) {
          gameState = MOVEUP;
          flashHeat = false;
          expLTnoPlay = millis();
        }
      }
      
      if (millis()  > expLTnoPlay + 10000) gameState  = EXPLODE;//you can't just sit on the take off pad...

      terrain(moveTerr);
      if (moveTerr && (collision == COLLGROUND || collision  == COLLMETEOR)) gameState = EXPLODE;//hit something freez the terrain and explode the ship
      if (moveTerr && heatF > 10) {
        heatF -= .6;
        heat = heatF;
      }
      drawLander(); 
   
      break;
    case MOVEUP://Lander is firing moving up
      if (digitalRead(upKey)) {
        gameState = MOVEDOWN;
      }
      if (!pupsLoaded && vLoc < 42) moveTerr = true;//after take  off move terraing again
      if (pupsLoaded && vLoc < 31) moveTerr = true;
      terrain(moveTerr);
      if (moveTerr && heatF < 30) {
        heatF += .2;
        heat = heatF;
      }   

      if (moveTerr && (collision == COLLGROUND || collision  == COLLMETEOR)) gameState = EXPLODE;      
      drawLanderFiring(); 

      if (heat > 25) flashHeat = true;      
      if(heat > 27) {
        overHeatCnt--;
        gameState = MOVEDOWN;//overheated and forced to descend
        overHeat = true;
      }

      if (fuelAmnt > 0) fuelAmnt--;
      else gameState = MOVEDOWN;
 
      break;
      case LANDED: //landedto to pick up the puppies
          moveTerr = false;
          landOk = true;     
          collision =  COLLNONE;     
          terrain(moveTerr);
          vLoc = 63 - landedY;
          drawLander(); 
          display.setCursor(50, 16);
          if (!pupsLoaded) display.println("SUCCESS!");
          if (pupsLoaded) display.println("PUPS SAVED!");

          display.display();
          delay(2000);
          if(!pupsLoaded) {
            gameState = LOADPUPS;
            dist = 795 + terrX - 8;
            for (int ld = 0; ld < 7; ld++)
                pupX[ld] = dist + (ld * 14);
          }            
          if (pupsLoaded) {
            gameState = UNLOADPUPS;
            dist = 1256 + terrX - 8;
            for (int ld = 0; ld < 7; ld++)
                pupX[ld] = (14 + 16) - (ld * 14);          
          }

        break;
      case LOADPUPS:
          moveTerr = false;
          terrain(moveTerr);
          vLoc = 63 - landedY;          
          drawLander(); 
          display.setCursor(50, 16);
          display.println("LOADING PUPS!");  
          
          pupY = 64 - 12 - 8;
          //walk the dog the distance from the wall to the lander.  Can vary depending on where the lander landed.

          for (int ld = 0; ld < 7; ld++){
              if (pupX[ld] < dist && pupX[ld] > 26) {
                display.drawBitmap(pupX[ld], pupY , puppy, puppy_W, puppy_H, 1);
                
              }
              pupX[ld]--;
          }
          if (pupX[6] < 14) {//last pup loaded
            expLTnoPlay = millis();
            gameState = RESET;
            fuelAmnt = MAXFUEL;
            heatF = 10;
            pupsLoaded = true;
            landNow = false;
            landOk = false;
            delay(2000);
          }
        break;      
      case UNLOADPUPS:
          moveTerr = false;
          terrain(moveTerr);
          vLoc = 63 - landedY;          
          drawLander(); 
          display.setCursor(30, 16);
          display.println("UNLOADING PUPS!");  
          
          pupY = 64 - 12 - 8;
          //walk the dog the distance from the lander to the wall.  Can vary depending on where the lander landed.

          for (int ld = 0; ld < 7; ld++){
              if (pupX[ld] > 30 && pupX[ld] < dist - 2) {
                display.drawBitmap(pupX[ld], pupY , puppyR, puppy_W, puppy_H, 1);
                
              }
              pupX[ld]++;
          }
          if (pupX[6] > dist + 2) {//last pup unloaded
            delay(2000);
            fullReset();
          }          
          break;
    case EXPLODE: //collision
        moveTerr = false;
        terrain(moveTerr);
        if (millis() > expLT){
          expLT = millis() + expLFrame;
          if (expLState < 9) expLState++;
        }
        switch (expLState){
          case 1:
              display.drawBitmap(6, vLoc - 8, expL1, expL_W, expL_H, 1);
            break;
          case 2:
              display.drawBitmap(6, vLoc - 8, expL2, expL_W, expL_H, 1);
            break;
          case 3:
              display.drawBitmap(6, vLoc - 8, expL3, expL_W, expL_H, 1);
            break;
          case 4:
              display.drawBitmap(6, vLoc - 8, expL4, expL_W, expL_H, 1);
            break;
          case 5:
              display.drawBitmap(6, vLoc - 8, expL5, expL_W, expL_H, 1);
            break;
          case 6:
              display.drawBitmap(6, vLoc - 8, expL6, expL_W, expL_H, 1);
            break;
          case 7:
              display.drawBitmap(6, vLoc - 8, expL7, expL_W, expL_H, 1);
            break;                                                
          case 8:
              display.drawBitmap(6, vLoc - 8, expL8, expL_W, expL_H, 1);
            break;
          case 9:
              //no display
              display.setTextSize(2);             // Draw 2X-scale text
              display.setCursor(1, 25);
              display.println("GAME OVER!");
              if (!digitalRead(upKey)){//reset game
                fullReset();
                delay(500);
              }
            break;
        }
      break;
  }
  display.display();
  delay(spd);
  Serial.println(String(terrX) +","  + String(gameState) + "," + String(expLState));
}

void dispHeat(){
    display.println("C");
    display.setCursor(120, 0);
    display.println("H");
    display.fillRect(88,0,heat,fuelH,SSD1306_WHITE); 
}

void terrain(bool mvTerr){
  if (millis() > terrX_T + terrMvT){
    if(mvTerr) terrX--;
    terrX_T = millis();
  }


  if (!refuelGo && mvTerr && fuelAmnt < 150) {
    refuelGo = true;
  }

  if (terrX == -350 || terrX == -1000) {
    meteorSpd = random(5,10);
    meteorGo = true;//always send after 2nd Mountain  range
  }

  if ((terrX < -650 && terrX > -660) or (terrX < -1135 && terrX > -1145)){
    display.setCursor(50, 16);
    display.println("Prep To Land");
  }
  if (!pupsLoaded && !landOk && terrX < -700 && terrX > -770 + 16 ){
    display.setCursor(50, 16);
    display.println("LAND NOW!");
    landNow = true;
  }
  else if (!pupsLoaded) landNow = false;

  if (pupsLoaded && !landOk && terrX < -1162 and terrX > -1232 + 16){
    display.setCursor(50, 16);
    display.println("LAND NOW!");
    landNow = true;
  }  
  else if (pupsLoaded) landNow = false;

  if (mvTerr && refuelGo && millis() > refuelT + refuelSpd){
    refuelT = millis();
    display.drawBitmap(refuelX,17,refuel, refuel_W, refuel_H, 1);

    refuelX-=2;
    if (refuelX < -20){
      refuelX = OFFSCREENX;
      refuelGo = false;

    }
  }

  if (mvTerr && meteorGo && millis() > meteorT + meteorSpd){
    meteorT = millis();
    display.drawBitmap(meteorX,17,meteor, meteor_W, meteor_H, 1);

    meteorX-=2;
    if (meteorX < -20){
      meteorX = 148;
      meteorGo = false;
    }
  }
  if (terrX >= -1280){
    display.drawBitmap(terrX,35,terrainAll, terrain_W, terrain_H, 1);
    if (mvTerr) collision = checkCollision();
    if (!landOk && collision == COLLLANDED) {
      gameState = LANDED;
    }
  }
 
}
void fullReset(){

        gameState = RESET;
        fuelAmnt =  MAXFUEL;
        expLState = 0;
        terrX =  RESETX;
        vLoc = RESETY;
        overHeat = false;
        overHeatCnt = OVERHEATMAX;
        flashHeat = false;
        meteorGo = true;
        refuelGo = false;
        meteorX = OFFSCREENX;
        refuelX = OFFSCREENX;
        heatF = 10;
        landNow = false;
        landOk - false;
        pupsLoaded = false;

}

int checkCollision(){
  //left of lander X is = 14
  //right of lander X is = 14 + lander_W
  //Y of lander bottom row is vLoc + lader_H
  //find any pixels from background that overlap the bottom row of the lander
  bool pix;

  if (refuelGo){
    int vY = vLoc;
    for (int ed = 0; ed < 16; ed++){//compare the leading  edge of lander array with the currently set pixels for the meteor
        pix = display.getPixel(14 + leadEdge[ed],vY++);
        if (pix && fuelAmnt < 500) {
          fuelAmnt += 200;//captured some fuel
          if(fuelAmnt > MAXFUEL) fuelAmnt = MAXFUEL;
          heatF -= 10;//cools the engines too
          refuelGo = false;
          refuelX = OFFSCREENX;
          return(COLLFUEL);
        }
    }
  }
  if (meteorGo){
    int vY = vLoc;
    for (int ed = 0; ed < 16; ed++){//compare the leading  edge of lander array with the currently set pixels for the meteor
        pix = display.getPixel(14 + leadEdge[ed],vY++);
        if (pix) return(COLLMETEOR);

    }
  }  
 

  for (int x = 14; x < 14 + lander_W; x++){
    pix = display.getPixel(x,vLoc + lander_H);
    if (pix && !landNow) return(COLLGROUND);
      if (pix && landNow) {
        return(COLLLANDED);
      }      
  }

  return(COLLNONE);

}
void drawLander(void) {


  if (vLoc < 44) vLoc+= 1;
  display.drawBitmap(
    14,
    vLoc,
    lander, lander_W, lander_H, 1);


}
void drawLanderFiring(void) {

  if (vLoc > 16) vLoc -= 1;
  display.drawBitmap(
    14,
    vLoc,
    lander_fire, lander_W, lander_H, 1);


}


